MODULE limistate
   !!======================================================================
   !!                     ***  MODULE  limistate  ***
   !!              Initialisation of diagnostics ice variables
   !!======================================================================
#if defined key_ice_lim
   !!----------------------------------------------------------------------
   !!   'key_ice_lim' :                                   LIM sea-ice model
   !!----------------------------------------------------------------------
   !!   lim_istate      :  Initialisation of diagnostics ice variables
   !!   lim_istate_init :  initialization of ice state and namelist read
   !!----------------------------------------------------------------------
   !! * Modules used
   USE phycst
   USE ocfzpt
   USE oce             ! dynamics and tracers variables
   USE dom_oce
   USE par_ice         ! ice parameters
   USE ice_oce         ! ice variables
   USE in_out_manager
   USE dom_ice
   USE ice
   USE lbclnk

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC lim_istate      ! routine called by lim_init.F90

   !! * Module variables
   REAL(wp) ::           & !!! ** init namelist (namiceini) **
      ttest  = 2.0  ,    &  ! threshold water temperature for initial sea ice
      hninn  = 0.5  ,    &  ! initial snow thickness in the north
      hginn  = 3.0  ,    &  ! initial ice thickness in the north
      alinn  = 0.05 ,    &  ! initial leads area in the north
      hnins  = 0.1  ,    &  ! initial snow thickness in the south
      hgins  = 1.0  ,    &  ! initial ice thickness in the south
      alins  = 0.1          ! initial leads area in the south

   REAL(wp)  ::          &  ! constant values
      zzero   = 0.e0  ,  &
      zone    = 1.e0
   !!----------------------------------------------------------------------
   !!   LIM 2.0 , UCL-LODYC-IPSL  (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE lim_istate
      !!-------------------------------------------------------------------
      !!                    ***  ROUTINE lim_istate  ***
      !!
      !! ** Purpose :   defined the sea-ice initial state
      !!
      !! ** Method  :   restart from a state defined in a binary file
      !!                or from arbitrary sea-ice conditions
      !!
      !! History :
      !!   2.0  !  01-04  (C. Ethe, G. Madec)  Original code
      !!--------------------------------------------------------------------
      !! * Local variables
      INTEGER  ::   ji, jj, jk   ! dummy loop indices
      REAL(wp) ::   zidto,    &  ! temporary scalar
         zs0, ztf, zbin
      REAL(wp), DIMENSION(jpi,jpj) ::   &
         ztn
      !--------------------------------------------------------------------

 
      CALL lim_istate_init     !  reading the initials parameters of the ice

      !-- Initialisation of sst,sss,u,v do i=1,jpi
      u_io(:,:)  = 0.e0       ! ice velocity in x direction
      v_io(:,:)  = 0.e0       ! ice velocity in y direction

      ! Initialisation at tn or -2 if ice
      DO jj = 1, jpj
         DO ji = 1, jpi
            zbin = MAX( 0., SIGN( 1., fzptn(ji,jj) - tn(ji,jj,1) ) )
            ztn(ji,jj) = ( (1.-zbin) * tn(ji,jj,1) - 2. * zbin + rt0 ) * tmask(ji,jj,1)
         END DO
      END DO


      u_io  (:,:) = 0.e0
      v_io  (:,:) = 0.e0
      sst_io(:,:) = ( nfice - 1 ) * ( tn(:,:,1) + rt0 )   ! use the ocean initial values
      sss_io(:,:) = ( nfice - 1 ) *   sn(:,:,1)           ! tricky trick *(nfice-1) !

      ! reference salinity 34psu
      zs0 = 34.e0
      ztf = ABS ( rt0 - 0.0575       * zs0                               &
               &                    + 1.710523e-03 * zs0 * SQRT( zs0 )   &
               &                    - 2.154996e-04 * zs0 *zs0          )

      !  tfu: Melting point of sea water
      tfu(:,:)  = ztf   
   
      DO jj = 1, jpj
         DO ji = 1, jpi
            !--- Criterion for presence (zidto=1) or absence (zidto=0) of ice
            zidto  = tms(ji,jj) * ( 1.0 - MAX(zzero, SIGN( zone, ztn(ji,jj) - tfu(ji,jj) - ttest) ) )

            IF( fcor(ji,jj) >= 0.e0 ) THEN     !--  Northern hemisphere.
               hicif(ji,jj)   = zidto * hginn
               frld(ji,jj)    = zidto * alinn + ( 1.0 - zidto ) * 1.0
               hsnif(ji,jj)   = zidto * hninn
            ELSE                               !---  Southern hemisphere.
               hicif(ji,jj)   = zidto * hgins
               frld(ji,jj)    = zidto * alins + ( 1.0 - zidto ) * 1.0
               hsnif(ji,jj)   = zidto * hnins
            ENDIF
         END DO
      END DO

      sist  (:,:)   = tfu(:,:)
      tbif  (:,:,1) = tfu(:,:)
      tbif  (:,:,2) = tfu(:,:)
      tbif  (:,:,3) = tfu(:,:)
      fsbbq (:,:)   = 0.e0
      qstoif(:,:)   = 0.e0
      u_ice (:,:)   = 0.e0
      v_ice (:,:)   = 0.e0
# if defined key_coupled
      albege(:,:)   = 0.8 * tms(:,:)
# endif

      !---  Moments for advection.             

      sxice (:,:)  = 0.e0   ;   sxsn (:,:)  = 0.e0   ;   sxa  (:,:)  = 0.e0
      syice (:,:)  = 0.e0   ;   sysn (:,:)  = 0.e0   ;   sya  (:,:)  = 0.e0
      sxxice(:,:)  = 0.e0   ;   sxxsn(:,:)  = 0.e0   ;   sxxa (:,:)  = 0.e0
      syyice(:,:)  = 0.e0   ;   syysn(:,:)  = 0.e0   ;   syya (:,:)  = 0.e0
      sxyice(:,:)  = 0.e0   ;   sxysn(:,:)  = 0.e0   ;   sxya (:,:)  = 0.e0

      sxc0  (:,:)  = 0.e0   ;   sxc1 (:,:)  = 0.e0   ;   sxc2 (:,:)  = 0.e0
      syc0  (:,:)  = 0.e0   ;   syc1 (:,:)  = 0.e0   ;   syc2 (:,:)  = 0.e0
      sxxc0 (:,:)  = 0.e0   ;   sxxc1(:,:)  = 0.e0   ;   sxxc2(:,:)  = 0.e0
      syyc0 (:,:)  = 0.e0   ;   syyc1(:,:)  = 0.e0   ;   syyc2(:,:)  = 0.e0
      sxyc0 (:,:)  = 0.e0   ;   sxyc1(:,:)  = 0.e0   ;   sxyc2(:,:)  = 0.e0

      sxst  (:,:)  = 0.e0
      syst  (:,:)  = 0.e0
      sxxst (:,:)  = 0.e0
      syyst (:,:)  = 0.e0
      sxyst (:,:)  = 0.e0

      !-- lateral boundary conditions
      CALL lbc_lnk( hicif, 'T', 1. )
      CALL lbc_lnk( frld , 'T', 1. )
!i bug frld = 1 over land
      frld(:,:) = tms(:,:) * frld(:,:) + ( 1. - tms(:,:) )   ! put 1 over land
!i
      CALL lbc_lnk( hsnif, 'T', 1. )
      CALL lbc_lnk( sist , 'T', 1. )
      DO jk = 1, jplayersp1
         CALL lbc_lnk(tbif(:,:,jk), 'T', 1. )
      END DO
      CALL lbc_lnk( fsbbq  , 'T', 1. )
      CALL lbc_lnk( qstoif , 'T', 1. )
      CALL lbc_lnk( sss_io , 'T', 1. )

   END SUBROUTINE lim_istate

   
   SUBROUTINE lim_istate_init
      !!-------------------------------------------------------------------
      !!                   ***  ROUTINE lim_istate_init  ***
      !!        
      !! ** Purpose :   Definition of initial state of the ice 
      !!
      !! ** Method  :   Read the namiceini namelist and check the parameter 
      !!       values called at the first timestep (nit000)
      !!
      !! ** input   :   Namelist namiceini
      !!
      !! history
      !!  8.5  ! 03-08 (C. Ethe) original code
      !!-------------------------------------------------------------------
      NAMELIST/namiceini/ ttest, hninn, hginn, alinn, hnins, hgins, alins
      !!-------------------------------------------------------------------

      ! Read Namelist namiceini 

      REWIND ( numnam_ice )
      READ   ( numnam_ice , namiceini )
      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'lim_istate_init : ice parameters inititialisation '
         WRITE(numout,*) '~~~~~~~~~~~~~~~'
         WRITE(numout,*) '         threshold water temp. for initial sea-ice    ttest      = ', ttest
         WRITE(numout,*) '         initial snow thickness in the north          hninn      = ', hninn
         WRITE(numout,*) '         initial ice thickness in the north           hginn      = ', hginn 
         WRITE(numout,*) '         initial leads area in the north              alinn      = ', alinn            
         WRITE(numout,*) '         initial snow thickness in the south          hnins      = ', hnins 
         WRITE(numout,*) '         initial ice thickness in the south           hgins      = ', hgins
         WRITE(numout,*) '         initial leads area in the south              alins      = ', alins
      ENDIF
            
   END SUBROUTINE lim_istate_init

#else
   !!----------------------------------------------------------------------
   !!   Default option :         Empty module          NO LIM sea-ice model
   !!----------------------------------------------------------------------
CONTAINS
   SUBROUTINE lim_istate          ! Empty routine
   END SUBROUTINE lim_istate
#endif

   !!======================================================================
END MODULE limistate
