MODULE closea
   !!======================================================================
   !!                       ***  MODULE  closea  ***
   !! Closed Seas  : 
   !!======================================================================

   !!----------------------------------------------------------------------
   !!   dom_clo    : modification of the ocean domain for closed seas cases
   !!   flx_clo    : Special handling of closed seas
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE in_out_manager  ! I/O manager
   USE ocesbc          ! ocean surface boundary conditions (fluxes)
   USE flxrnf          ! runoffs
   USE lib_mpp         ! distributed memory computing library
   USE lbclnk          ! ???

   IMPLICIT NONE
   PRIVATE

   !! * Accessibility
   PUBLIC dom_clo      ! routine called by dom_init
   PUBLIC flx_clo      ! routine called by step

   !! * Share module variables
   INTEGER, PUBLIC, PARAMETER ::   &  !:
      jpncs   = 4               !: number of closed sea
   INTEGER, PUBLIC ::          & !!: namclo : closed seas and lakes
      nclosea =  0                !: = 0 no closed sea or lake
      !                           !  = 1 closed sea or lake in the domain
   INTEGER, PUBLIC, DIMENSION (jpncs) ::   &  !:
      ncstt,           &  !: Type of closed sea
      ncsi1, ncsj1,    &  !: closed sea limits                                                                 
      ncsi2, ncsj2,    &  !: 
      ncsnr               !: number of point where run-off pours
   INTEGER, PUBLIC, DIMENSION (jpncs,4) ::   &
      ncsir, ncsjr        !: Location of run-off

   !! * Module variable
   REAL(wp), DIMENSION (jpncs+1) ::   &
      surf               ! closed sea surface

   !! * Substitutions
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!  OPA 9.0 , LODYC-IPSL (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE dom_clo
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE dom_clo  ***
      !!        
      !! ** Purpose :   Closed sea domain initialization
      !!
      !! ** Method  :   if a closed sea is located only in a model grid point
      !!      just the thermodynamic processes are applied.
      !!
      !! ** Action :   ncsi1(), ncsj1() : south-west closed sea limits (i,j)
      !!               ncsi2(), ncsj2() : north-east Closed sea limits (i,j)
      !!               ncsir(), ncsjr() : Location of runoff
      !!               ncsnr            : number of point where run-off pours
      !!               ncstt            : Type of closed sea
      !!                                  =0 spread over the world ocean
      !!                                  =2 put at location runoff
      !!
      !! History :
      !!        !  01-04  (E. Durand)  Original code
      !!   8.5  !  02-06  (G. Madec)  F90: Free form and module
      !!----------------------------------------------------------------------
      !! * Local variables
      INTEGER ::   jc            ! dummy loop indices
      !!----------------------------------------------------------------------
      
      IF(lwp) WRITE(numout,*)
      IF(lwp) WRITE(numout,*)'dom_clo : closed seas '
      IF(lwp) WRITE(numout,*)'~~~~~~~'

      ! initial values
      ncsnr(:) = 1  ;  ncsi1(:) = 1  ;  ncsi2(:) = 1  ;  ncsir(:,:) = 1
      ncstt(:) = 0  ;  ncsj1(:) = 1  ;  ncsj2(:) = 1  ;  ncsjr(:,:) = 1

      ! set the closed seas (in data domain indices)
      ! -------------------

      IF( cp_cfg == "orca" ) THEN
   
         SELECT CASE ( jp_cfg )
         !                                           ! =======================
         CASE ( 2 )                                  !  ORCA_R2 configuration
            !                                        ! =======================

            !                                            ! Caspian Sea
            ncsnr(1)   =   1  ;  ncstt(1)   =   0           ! spread over the globe
            ncsi1(1)   =  11  ;  ncsj1(1)   = 103
            ncsi2(1)   =  17  ;  ncsj2(1)   = 112
            ncsir(1,1) =   1  ;  ncsjr(1,1) =   1 
            !                                            ! Great North American Lakes
            ncsnr(2)   =   1  ;  ncstt(2)   =   2           ! put at St Laurent mouth
            ncsi1(2)   =  97  ;  ncsj1(2)   = 107
            ncsi2(2)   = 103  ;  ncsj2(2)   = 111
            ncsir(2,1) = 110  ;  ncsjr(2,1) = 111
            !                                            ! Black Sea 1 : west part of the Black Sea 
            ncsnr(3)   = 1    ; ncstt(3)   =   2            !            (ie west of the cyclic b.c.)
            ncsi1(3)   = 174  ; ncsj1(3)   = 107            ! put in Med Sea
            ncsi2(3)   = 181  ; ncsj2(3)   = 112
            ncsir(3,1) = 171  ; ncsjr(3,1) = 106 
            !                                            ! Black Sea 2 : est part of the Black Sea 
            ncsnr(4)   =   1  ;  ncstt(4)   =   2           !               (ie est of the cyclic b.c.)
            ncsi1(4)   =   2  ;  ncsj1(4)   = 107           ! put in Med Sea
            ncsi2(4)   =   6  ;  ncsj2(4)   = 112
            ncsir(4,1) = 171  ;  ncsjr(4,1) = 106 

            !                                        ! =======================
         CASE ( 4 )                                  !  ORCA_R4 configuration
            !                                        ! =======================

            !                                            ! Caspian Sea
            ncsnr(1)   =  1  ;  ncstt(1)   =  0  
            ncsi1(1)   =  4  ;  ncsj1(1)   = 53 
            ncsi2(1)   =  4  ;  ncsj2(1)   = 56
            ncsir(1,1) =  1  ;  ncsjr(1,1) =  1
            !                                            ! Great North American Lakes
            ncsnr(2)   =  1  ;  ncstt(2)   =  2 
            ncsi1(2)   = 49  ;  ncsj1(2)   = 55
            ncsi2(2)   = 51  ;  ncsj2(2)   = 56
            ncsir(2,1) = 57  ;  ncsjr(2,1) = 55
            !                                            ! Black Sea
            ncsnr(3)   =  4  ;  ncstt(3)   =  2  
            ncsi1(3)   = 88  ;  ncsj1(3)   = 55 
            ncsi2(3)   = 91  ;  ncsj2(3)   = 56
            ncsir(3,1) = 86  ;  ncsjr(3,1) = 53
            ncsir(3,2) = 87  ;  ncsjr(3,2) = 53 
            ncsir(3,3) = 86  ;  ncsjr(3,3) = 52 
            ncsir(3,4) = 87  ;  ncsjr(3,4) = 52
            !                                            ! Baltic Sea
            ncsnr(4)   =  1  ;  ncstt(4)   =  2
            ncsi1(4)   = 75  ;  ncsj1(4)   = 59
            ncsi2(4)   = 76  ;  ncsj2(4)   = 61
            ncsir(4,1) = 84  ;  ncsjr(4,1) = 59 

         END SELECT

      ENDIF

      ! convert the position in local domain indices
      ! --------------------------------------------
      DO jc = 1, jpncs
         ncsi1(jc)   = mi0( ncsi1(jc) )
         ncsj1(jc)   = mj0( ncsj1(jc) )

         ncsi2(jc)   = mi1( ncsi2(jc) )   
         ncsj2(jc)   = mj1( ncsj2(jc) )  
      END DO
         

   END SUBROUTINE dom_clo


   SUBROUTINE flx_clo( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE flx_clo  ***
      !!                    
      !! ** Purpose :   Special handling of closed seas
      !!
      !! ** Method  :   Water flux is forced to zero over closed sea
      !!      Excess is shared between remaining ocean, or
      !!      put as run-off in open ocean.
      !!
      !! ** Action :
      !!
      !! History :
      !!   8.2  !  00-05  (O. Marti)  Original code
      !!   8.5  !  02-07  (G. Madec)  Free form, F90
      !!----------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT (in) :: kt

      !! * Local declarations
      REAL(wp), DIMENSION (jpncs) :: zemp
      INTEGER  :: ji, jj, jc, jn
      REAL(wp) :: zze2
      !!----------------------------------------------------------------------

      ! 1 - Initialisation
      ! ------------------

      IF( kt == nit000 ) THEN 
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*)'flx_clo : closed seas '
         IF(lwp) WRITE(numout,*)'~~~~~~~'

         ! Total surface of ocean
         surf(jpncs+1) = SUM( e1t(:,:) * e2t(:,:) * tmask_i(:,:) )

         DO jc = 1, jpncs
            surf(jc) =0.e0
            DO jj = ncsj1(jc), ncsj2(jc)
               DO ji = ncsi1(jc), ncsi2(jc)
                  ! surface of closed seas
                  surf(jc) = surf(jc) + e1t(ji,jj)*e2t(ji,jj)*tmask_i(ji,jj)
                  ! upstream in closed seas
                  upsadv(ji,jj) = 0.5
               END DO 
            END DO 
            ! upstream at closed sea outflow
            IF( ncstt(jc) >= 1 ) THEN 
                DO jn = 1, 4
                  ji = mi0( ncsir(jc,jn) )
                  jj = mj0( ncsjr(jc,jn) )
                  upsrnfh(ji,jj) = MAX( upsrnfh(ji,jj), 1.0 )
                END DO 
            ENDIF 
         END DO 
         IF( lk_mpp )   CALL mpp_sum ( surf, jpncs+1 )       ! mpp: sum over all the global domain

         IF(lwp) WRITE(numout,*)'     Closed sea surfaces'
         DO jc = 1, jpncs
            IF(lwp) WRITE(numout,FMT='(1I3,4I4,5X,F16.2)')    &
                jc, ncsi1(jc), ncsi2(jc), ncsj1(jc), ncsj2(jc), surf(jc)
         END DO

         ! jpncs+1 : surface of sea, closed seas excluded
         DO jc = 1, jpncs
            surf(jpncs+1) = surf(jpncs+1) - surf(jc)
         END DO           
 
      ENDIF

      ! 2 - Computation
      ! ---------------
      zemp = 0.e0

      DO jc = 1, jpncs
         DO jj = ncsj1(jc), ncsj2(jc)
            DO ji = ncsi1(jc), ncsi2(jc)
               zemp(jc) = zemp(jc) + e1t(ji,jj) * e2t(ji,jj) * emp(ji,jj) * tmask_i(ji,jj)
            END DO  
         END DO 
      END DO
      IF( lk_mpp )   CALL mpp_sum ( zemp , jpncs )       ! mpp: sum over all the global domain

      IF( cp_cfg == "orca" .AND. jp_cfg == 2 ) THEN      ! Black Sea case for ORCA_R2 configuration
         zze2    = ( zemp(3) + zemp(4) ) / 2.
         zemp(3) = zze2
         zemp(4) = zze2
      ENDIF

      DO jc = 1, jpncs

         IF( ncstt(jc) == 0 ) THEN 
            ! water/evap excess is shared by all open ocean
            emp (:,:) = emp (:,:) + zemp(jc) / surf(jpncs+1)
            emps(:,:) = emps(:,:) + zemp(jc) / surf(jpncs+1)
         ELSEIF( ncstt(jc) == 1 ) THEN 
            ! Excess water in open sea, at outflow location, excess evap shared
            IF ( zemp(jc) <= 0.e0 ) THEN 
                DO jn = 1, ncsnr(jc)
                  ji = mi0(ncsir(jc,jn))
                  jj = mj0(ncsjr(jc,jn)) ! Location of outflow in open ocean
                  IF (      ji > 1 .AND. ji < jpi   &
                      .AND. jj > 1 .AND. jj < jpj ) THEN 
                      emp (ji,jj) = emp (ji,jj) + zemp(jc) /   &
                         (FLOAT(ncsnr(jc)) * e1t(ji,jj) * e2t(ji,jj))
                      emps(ji,jj) = emps(ji,jj) + zemp(jc) /   &
                          (FLOAT(ncsnr(jc)) * e1t(ji,jj) * e2t(ji,jj))
                  END IF 
                END DO 
            ELSE 
                emp (:,:) = emp (:,:) + zemp(jc) / surf(jpncs+1)
                emps(:,:) = emps(:,:) + zemp(jc) / surf(jpncs+1)
            ENDIF
         ELSEIF( ncstt(jc) == 2 ) THEN 
            ! Excess e-p+r (either sign) goes to open ocean, at outflow location
            IF(      ji > 1 .AND. ji < jpi    &
               .AND. jj > 1 .AND. jj < jpj ) THEN 
                DO jn = 1, ncsnr(jc)
                  ji = mi0(ncsir(jc,jn))
                  jj = mj0(ncsjr(jc,jn)) ! Location of outflow in open ocean
                  emp (ji,jj) = emp (ji,jj) + zemp(jc)   &
                      / (FLOAT(ncsnr(jc)) *  e1t(ji,jj) * e2t(ji,jj) )
                  emps(ji,jj) = emps(ji,jj) + zemp(jc)   &
                      / (FLOAT(ncsnr(jc)) *  e1t(ji,jj) * e2t(ji,jj) )
                END DO 
            ENDIF 
         ENDIF 

         DO jj = ncsj1(jc), ncsj2(jc)
            DO ji = ncsi1(jc), ncsi2(jc)
               emp (ji,jj) = emp (ji,jj) - zemp(jc) / surf(jc)
               emps(ji,jj) = emps(ji,jj) - zemp(jc) / surf(jc)
            END DO  
         END DO 

      END DO 


      ! 5. Boundary condition on emp and emps
      ! -------------------------------------
      CALL lbc_lnk( emp , 'T', 1. )
      CALL lbc_lnk( emps, 'T', 1. )

   END SUBROUTINE flx_clo

   !!======================================================================
END MODULE closea
