MODULE dynzdf_exp
   !!==============================================================================
   !!                     ***  MODULE  dynzdf_exp  ***
   !! Ocean dynamics:  vertical component(s) of the momentum mixing trend
   !!==============================================================================

   !!----------------------------------------------------------------------
   !!   dyn_zdf_exp  : update the momentum trend with the vertical diffu-
   !!                  sion using an explicit time-stepping scheme.
   !!----------------------------------------------------------------------
   !! * Modules used
   USE oce             ! ocean dynamics and tracers
   USE dom_oce         ! ocean space and time domain
   USE phycst          ! physical constants
   USE zdf_oce         ! ocean vertical physics
   USE in_out_manager  ! I/O manager
   USE taumod          ! surface ocean stress
   USE trddyn_oce     ! dynamics trends diagnostics variables

   IMPLICIT NONE
   PRIVATE

   !! * Routine accessibility
   PUBLIC dyn_zdf_exp    ! called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LODYC-IPSL  (2003)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE dyn_zdf_exp( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE dyn_zdf_exp  ***
      !!                   
      !! ** Purpose :   Compute the trend due to the vert. momentum diffusion
      !!
      !! ** Method  :   Explicit forward time stepping with a time splitting
      !!      technique. The vertical diffusion of momentum is given by:
      !!         diffu = dz( avmu dz(u) ) = 1/e3u dk+1( avmu/e3uw dk(ub) )
      !!      Surface boundary conditions: wind stress input
      !!      Bottom boundary conditions : bottom stress (cf zdfbfr.F90)
      !!      Add this trend to the general trend ua :
      !!         ua = ua + dz( avmu dz(u) )
      !!
      !! ** Action : - Update (ua,va) with the vertical diffusive trend
      !!             - Save the trends in (utrd,vtrd) ('key_diatrends')
      !!
      !! History :
      !!        !  90-10  (B. Blanke)  Original code
      !!        !  97-05  (G. Madec)  vertical component of isopycnal
      !!   8.5  !  02-08  (G. Madec)  F90: Free form and module
      !!---------------------------------------------------------------------
      !! * Arguments
      INTEGER, INTENT( in ) ::   kt       ! ocean time-step index

      !! * Local declarations
      INTEGER ::   ji, jj, jk, jl         ! dummy loop indices
      REAL(wp) ::   &
         zrau0r, zlavmr, z2dt, zua, zva   ! temporary scalars
      REAL(wp), DIMENSION(jpi,jpk) ::   &
         zwx, zwy, zwz, zww               ! temporary workspace arrays
#if defined key_trddyn || defined key_trd_vor
      INTEGER ::   &
         ikbu, ikbum1 , ikbv, ikbvm1      ! temporary integers
#endif
      !!----------------------------------------------------------------------

      IF( kt == nit000 ) THEN
         IF(lwp) WRITE(numout,*)
         IF(lwp) WRITE(numout,*) 'dyn_zdf_exp : vertical momentum diffusion explicit operator'
         IF(lwp) WRITE(numout,*) '~~~~~~~~~~~ '
      ENDIF

      ! Local constant initialization
      ! -----------------------------
      zrau0r = 1. / rau0                                   ! inverse of the reference density
      zlavmr = 1. / float( n_zdfexp )                      ! inverse of the number of sub time step
      z2dt = 2. * rdt                                      ! Leap-frog environnement
      IF( neuler == 0 .AND. kt == nit000 )   z2dt = rdt    ! Euler time stepping when starting from rest

      !                                                ! ===============
      DO jj = 2, jpjm1                                 !  Vertical slab
         !                                             ! ===============

         ! Surface boundary condition
         DO ji = 2, jpim1
            zwy(ji,1) = taux(ji,jj) * zrau0r
            zww(ji,1) = tauy(ji,jj) * zrau0r
         END DO  

         ! Initialization of x, z and contingently trends array
         DO jk = 1, jpk
            DO ji = 2, jpim1
               zwx(ji,jk) = ub(ji,jj,jk)
               zwz(ji,jk) = vb(ji,jj,jk)
#if defined key_trddyn || defined key_trd_vor
               utrd(ji,jj,jk,7) = ua(ji,jj,jk)
               vtrd(ji,jj,jk,7) = va(ji,jj,jk)
#endif
            END DO  
         END DO  

         ! Time splitting loop
         DO jl = 1, n_zdfexp

            ! First vertical derivative
            DO jk = 2, jpk
               DO ji = 2, jpim1
                  zwy(ji,jk) = avmu(ji,jj,jk) * ( zwx(ji,jk-1) - zwx(ji,jk) ) / fse3uw(ji,jj,jk) 
                  zww(ji,jk) = avmv(ji,jj,jk) * ( zwz(ji,jk-1) - zwz(ji,jk) ) / fse3vw(ji,jj,jk)
               END DO  
            END DO  

            ! Second vertical derivative and trend estimation at kt+l*rdt/n_zdfexp
            DO jk = 1, jpkm1
               DO ji = 2, jpim1
                  zua = zlavmr*( zwy(ji,jk) - zwy(ji,jk+1) ) / fse3u(ji,jj,jk)
                  zva = zlavmr*( zww(ji,jk) - zww(ji,jk+1) ) / fse3v(ji,jj,jk)
                  ua(ji,jj,jk) = ua(ji,jj,jk) + zua
                  va(ji,jj,jk) = va(ji,jj,jk) + zva

                  zwx(ji,jk) = zwx(ji,jk) + z2dt*zua*umask(ji,jj,jk)
                  zwz(ji,jk) = zwz(ji,jk) + z2dt*zva*vmask(ji,jj,jk)
               END DO  
            END DO  

         END DO  

#if defined key_trddyn || defined key_trd_vor
         ! diagnose the vertical diffusive momentum trends

         ! save the total vertical momentum diffusive trend
         DO jk = 1, jpkm1
            DO ji = 2, jpim1
               utrd(ji,jj,jk,7) = ua(ji,jj,jk) - utrd(ji,jj,jk,7)
               vtrd(ji,jj,jk,7) = va(ji,jj,jk) - vtrd(ji,jj,jk,7)
            END DO 
         END DO 
 
         ! subtract and save surface and momentum fluxes
         DO ji = 2, jpim1
            ! save the surface momentum fluxes 
            tautrd(ji,jj,1) = zwy(ji,1) / fse3u(ji,jj,1)
            tautrd(ji,jj,2) = zww(ji,1) / fse3v(ji,jj,1)
            ! save bottom friction momentum fluxes 
            ikbu   = MIN( mbathy(ji+1,jj), mbathy(ji,jj) )
            ikbum1 = MAX( ikbu-1, 1 )
            ikbv   = MIN( mbathy(ji,jj+1), mbathy(ji,jj) )
            ikbvm1 = MAX( ikbv-1, 1 )
            tautrd(ji,jj,3) = avmu(ji,jj,ikbu) * zwx(ji,ikbum1)   &
                            / ( fse3u(ji,jj,ikbum1) * fse3uw(ji,jj,ikbu) )
            tautrd(ji,jj,4) = avmv(ji,jj,ikbv) * zwz(ji,ikbvm1)   &
                            / ( fse3v(ji,jj,ikbvm1) * fse3vw(ji,jj,ikbv) )
            ! subtract surface forcing and bottom friction trend from vertical
            ! diffusive momentum trend
            utrd(ji,jj,1     ,7) = utrd(ji,jj,1     ,7) - tautrd(ji,jj,1)
            utrd(ji,jj,ikbum1,7) = utrd(ji,jj,ikbum1,7) - tautrd(ji,jj,3)
            vtrd(ji,jj,1     ,7) = vtrd(ji,jj,1     ,7) - tautrd(ji,jj,2)
            vtrd(ji,jj,ikbvm1,7) = vtrd(ji,jj,ikbvm1,7) - tautrd(ji,jj,4)
         END DO
#endif
         !                                             ! ===============
      END DO                                           !   End of slab
      !                                                ! ===============

   END SUBROUTINE dyn_zdf_exp

   !!==============================================================================
END MODULE dynzdf_exp
