C $Id$
CCC-----------------------------------------------------------------
CCC
CCC                     ROUTINE trcini.hamocc3.h
CCC                     ************************
CCC
CCC  PURPOSE :
CCC  ---------
CCC     Initialisation of HAMOCC3 biological and chemical variables
CCC
CC   METHOD :
CC   -------
CC         1) SET CONSTANTS FOR CARBONATE CHEMISTRY AS DESCRIBED IN
CC            IN BROECKER ET AL. (1982, GEOSECS) AND EDMOND A. GIESKES
CC            (1970)
CC         2) INITIATE [CO3--] AND PH-VALUE BY ITERATION
CC            (NEWTON-RAPHSON METHOD FOR SOLVING NONLINEAR SIMULTANEOUS
CC             EQUATIONS, SEE E.G. SCARBOROUGH, J. (1958))
CC
CC      This sub-routine merges previous initia2.F bioini.F bioini2.F
CC
CC   INPUT :
CC   -----
CC      common
CC              all the common defined in opa 
CC
CC
CC   OUTPUT :                   : no
CC   ------
CC
CC   WORKSPACE :
CC   ---------
CC
CC   EXTERNAL :
CC   --------
CC      SEAICE
CC      WSJCD
CC      WSEK
CC      RHO
CC
CC   MODIFICATIONS:
CC   --------------
CC      original  : 1988-07  E. MAIER-REIMER      MPI HAMBURG
CC      additions : 1999-10  O. Aumont and C. Le Quere
CC      additions : 2001-03  0. Aumont and E. Kastenare : Changes in computation
CC                           of the export profiles for silicate and calcite
CC
CC   REFERENCE for biology:
CC   ----------------------
CC
CC         DEGENS, E.T, S. KEMPE, AND A. SPITZY (1984)
CC         CARBON DIOXIDE: A BIOGEOCHEMICAL PORTRAIT.
CC         IN: THE HANDBOOK OF ENVIRONMENTAL CHEMISTRY, VOLUME 1/
CC         PART C, O. HUTZINGER, ED., SPRINGER-VERLAG, BERLIN,
CC         HEIDELBERG, PP. 127-215.
CC
CC         DUGDALE. R.C. (1967)
CC         NUTRIENT LIMITATION IN THE SEA: DYNAMICS, IDENTIFICATION
CC         AND SIGNIFICANCE.
CC         LIMNOLOGY AND OCEANOGRAPHY, VOL.12, 685-695.
CC
CC         PARSONS, T.R., AND M. TAKAHASHI (1973)
CC         BIOLOGICAL OCEANOGRAPHIC PROCESSES.
CC         PERGAMON PRESS, 186 PP.
CC
CC         TAKAHASHI, T., W.S. BROECKER, AND S. LANGER (1985)
CC         REDFIELD RATIO BASED ON CHEMICAL DATA FROM ISOPYCNAL
CC         SURFACES.
CC         JOURNAL OF GEOPHYSICAL RESEARCH, 90(C4), 6907-6924.
CC
CC    REFERENCE for chemistry:
CC    -----------------------
CC
CC         BERNER, R. A. (1976)
CC         THE SOLUBILITY OF CALCITE AND ARAGONITE IN SEA WATER
CC         AT ATMOSPHERIC PRESSURE AND 34.5 O/OO SALINITY.
CC         AMERICAN JOURNAL OF SCIENCE, VOL. 276, 713-730.
CC         (K'SP(ARAGONITE)=1.45 K'SP(CALCITE))
CC
CC         BROECKER, W.S., D.W. SPENCER, AND H. CRAIG (1982)
CC         GEOSECS PACIFIC EXPEDITION. VOL. 3.. HYDROGRAPHIC DATA
CC         1973-1974, SUPERINTENDANT OF DOCUMENTS, U.S. GOVERNMENT
CC         PRINTING OFFICE, WASHINGTON, D.C., 137 PP..
CC
CC         CULBERSON, C.H., AND R.M. PYTKOWICZ (1968)
CC         EFFECT ON PRESSURE ON CARBONIC ACID, BORIC ACID AND THE PH
CC         IN SEA WATER.
CC         LIMNOLOGY AND OCEANOGRAPHY, VOL. 13, 403-417.
CC
CC         DICKSON, A.G., AND J.P. RILEY (1979)
CC         THE ESTIMATION OF ACID DISSOCIATION CONSTANTS IN SEAWATER
CC         MEDIA FROM POTENTIOMETRIC TITRATIONS WITH STRONG BASE.
CC         I. THE IONIC PRODUCT OF WATER - KW.
CC         MARINE CHEMISTRY, VOL. 7, 89-99.
CC
CC         EDMOND, J.M., AND J.M.T.M. GIESKES (1970)
CC         ON THE CALCULATION OF THE DEGREE OF SATURATION OF SEA WATER
CC         WITH RESPECT TO CALCIUM CARBONATE UNDER IN SITU CONDITIONS.
CC         GEOCHIM. ET COSMOCHIM. ACTA, 34, 1261-1291.
CC
CC         INGLE, S.E. (1800)
CC         SOLUBILITY OF CALCITE IN THE OCEAN.
CC         MARINE CHEMISTRY, VOL. 3, 301-319.
CC
CC         INGLE, S.E., C.H. CULBERSON, J.E. HAWLEY, AND R.M. PYTKOWICZ
CC         (1973) THE SOLUBILITY OF CALCITE IN SEAWATER AT ATMOSPHERIC
CC         PRESSURE AND 35 O/OO SALINITY.
CC         MARINE CHEMISTRY, VOL. 1, 295-307.
CC
CC         RILEY, J. P., AND G. SKIRROW, EDS. (1965)
CC         CHEMICAL OCEANOGRAPHY. VOL. 1, 712 PP., ACADEMIC PRESS,
CC         LONDON A. NEW YORK.
CC
CC         SCARBOROUGH, J. (1958) NUMERICAL MATHEMATICAL ANALYSIS.
CC         OXFORD UNIVERSITY PRESS, LONDON, 4TH ED., 576 PP..
CC
CC         WEISS, R. F. (1970) THE SOLUBILITY OF NITROGEN
CC         OXYGEN AND ARGON IN WATER AND SEAWATER.
CC         DEEP-SEA RESEARCH, VOL. 17, 721-735.
CC
CC         WEISS, R. F. (1974)
CC         CARBON DIOXIDE IN WATER AND SEAWATER: THE SOLUBILITY OF A
CC         NON IDEAL GAS. MARINE CHEMISTRY, VOL. 2, 203-215.
CC
CC         WOOSTER, W.S., A.J. LEE, AND G. DIETRICH (1969)
CC         REDEFINITION OF SALINITY. Z. GEOPHYS., VOL.35, 611-613.
CC
CC         BROECKER, W.S., D.W. SPENCER, AND H. CRAIG (1982)
CC         GEOSECS PACIFIC EXPEDITION. VOL. 3.. HYDROGRAPHIC DATA
CC         1973-1974, SUPERINTENDANT OF DOCUMENTS, U.S. GOVERNMENT
CC         PRINTING OFFICE, WASHINGTON, D.C., 137 PP..
CC
CCC---------------------------------------------------------------------
CC local declarations
CC ==================
      INTEGER mo,ke1

      REAL zdm0(jpi,jpj,jpk),zds0(jpi,jpj,jpk)
      REAL zdc0(jpi,jpj,jpk),zrrc(jpi,jpj)
      REAL zrro(jpi,jpj), zrrs(jpi,jpj), tobbb, tob, tun
      REAL zfluo, zfluu
      REAL ztest
C
C
C
C
C
C 1. initialization
C -----------------
C
C  initialisation of local variables
C
      rfact = rdttra(1) * float(ndttrc)
      rfactr = 1./rfact
      WRITE(numout,*) ' Tracer time step=',rfact,' rdt=',rdt
C
CC
CC Initialize ice cover
CC
CC----------------------------------------------------------------------
C
      IF (icice .eq. 0) THEN
          WRITE (numout,*) ' No Sea Ice (default value) '
          cicemo = 0.0
      ELSEIF (icice .eq. 1) THEN
          WRITE (numout,*) ' Sea Ice from Reynolds surface data'
      ELSEIF (icice .eq. 2) THEN
          WRITE (numout,*) ' Sea Ice coverage from Walsh and Zwally'
          CALL h3cice
      ELSEIF (icice .eq. 3) THEN
          WRITE (numout,*) ' Sea Ice Coverage from coupled run'
      ENDIF
c
CC----------------------------------------------------------------------
CC
CC Initialize gas exchange
CC
CC----------------------------------------------------------------------
C
C
      IF (igaswind .eq. 0) THEN
          WRITE (numout,*) ' No winds read'
C
      ELSEIF (igaswind .eq. 1) THEN
C
C Hellerman and Rosenstein according to Jean-Claude Dutay
C
          WRITE (numout,*)' Interannual wind speed'
C
      ELSEIF (igaswind .eq. 2) THEN
C
C Esbensen and Kushnir according to Bonnie Samuels
C
          WRITE (numout,*)' Wind speed from Esbensen & Kushnir: '
          CALL h3cwin
          DO mo=1,12
#if defined key_vpp
            CALL READ2S(900,kgwanmo(1,1,mo),jpi,jpj)
#else
            READ (900) kgwanmo(:,:,mo)
#endif
          ENDDO
C
      ELSEIF (igaswind .eq. 3) THEN
C
          WRITE (numout,*) 'Wind speed from coupled run'
C
C   In case of offline simulations forced by coupled run output
C
      ELSE
C
          WRITE (numout,*) ' Variable igaswind invalid!'
          WRITE (numout,*) ' Rerun model with igaswind = 0, 1, 2, 3'
          STOP
C
      ENDIF
C
C
C
CC----------------------------------------------------------------------
CC
CC Initialize biological variables 
CC
CC----------------------------------------------------------------------
C
      spocri = 0.003
      cristl = 0.
      tobox  = 50.
      prorca = 0.
      prcaca = 0.
      silpro = 0.
#if defined key_trc_p3zd
      sinking = 0.
      nu = 0.
      tortz = 0.
      grazp = 0.
      grazpoc = 0.
      tortp = 0.
      respp = 0.
      respz = 0.
      dipn = 55.
#endif
C
C Set biological ratios
C ---------------------
C
      rno3   = (16.+2.)/122.
      po4r   = 1./122.
      o2ut   = 172./122.
      sio2r  = 0.75
C
C Set fractionation factors of 13C, 14C
C -------------------------------------
C
      pdb     = 0.011112
      plafr13 = 0.980
C
C Set e-folding depth for penetration of newly formed biogenic poc and
c caco3 [cm]
C ----------------------------------------------------------------------
c ---------
C
C
#if defined key_off_degrad
#    if defined key_vpp
      CALL READ3S(903,dmin3,jpi,jpj,jpk)
      CALL READ3S(903,diss3,jpi,jpj,jpk)
      CALL READ3S(903,disc3,jpi,jpj,jpk)
#    else
      READ(903) dmin3
      READ(903) diss3
      READ(903) disc3
#    endif
      WRITE (numout,*) 'OK for read of deg.min'
#else
C
C Calculate vertical distribution of newly formed biogenic poc 
C and caco3 particles in the water column in the case of 
C max. possible bottom depth
C ------------------------------------------------------------
C
C
      zrro   = 1.
      zrrs   = 1.
      zrrc   = 1.

      zdm0=0.
      zdc0=0.
      zds0=0.

      DO jk = jpkb+1,jpkm1
        DO jj = 1,jpj
          DO ji = 1,jpi

        tun = fsdepw(ji,jj,jk+1)
        tob = fsdepw(ji,jj,jk)
        tobbb=fsdepw(ji,jj,jpkb+1)
        zfluo = (1.E2/tob)**fluexp
        zfluu = (1.E2/tun)**fluexp
        IF (zfluo.gt.1.) zfluo = 1.
        zdm0(ji,jj,jk) = zfluo-zfluu
        zdc0(ji,jj,jk) = exp(-(tob-tobbb)/pendec)-
     &                   exp(-(tun-tobbb)/pendec)
        zds0(ji,jj,jk) = exp(-(tob-tobbb)/pendes)-
     &                   exp(-(tun-tobbb)/pendes)
        zrrs(ji,jj) = zrrs(ji,jj)-zds0(ji,jj,jk)
        zrro(ji,jj) = zrro(ji,jj)-zdm0(ji,jj,jk)
        zrrc(ji,jj) = zrrc(ji,jj)-zdc0(ji,jj,jk)
           END DO
         END DO
       END DO

        DO jj = 1,jpj
          DO ji = 1,jpi
      zdc0(ji,jj,jpk) = zrrc(ji,jj)
      zdm0(ji,jj,jpk) = zrro(ji,jj)
      zds0(ji,jj,jpk) = zrrs(ji,jj)
          END DO
        END DO
C
C Calculate vertical distribution of newly formed biogenic poc and 
C  caco3 particles in the water column with realistic topography (lowest
c 
C  wet layer contains total fraction, which has passed the upper layers)
C ----------------------------------------------------------------------
C
      dminl = 0.
      dissl = 0.
      discl = 0.
C
      dmin3 = zdm0
      diss3 = zds0
      disc3 = zdc0
C
      DO jk = 1,jpk
        DO jj = 1,jpj
          DO ji = 1,jpi
            IF(tmask(ji,jj,jk).eq.0.) THEN

                dminl(ji,jj) = dminl(ji,jj)+dmin3(ji,jj,jk)
                dissl(ji,jj) = dissl(ji,jj)+diss3(ji,jj,jk)
                discl(ji,jj) = discl(ji,jj)+disc3(ji,jj,jk)
                dmin3(ji,jj,jk) = 0.0
                diss3(ji,jj,jk) = 0.0
                disc3(ji,jj,jk) = 0.0
C
            ENDIF
          ENDDO
        ENDDO
      ENDDO
C
C
      DO jj = 1,jpj
        DO ji = 1,jpi
          IF (tmask(ji,jj,1).eq.0.) THEN
              dmin3(ji,jj,1) = 0.
              diss3(ji,jj,1) = 0.
              disc3(ji,jj,1) = 0.
          ENDIF
        ENDDO
      ENDDO
#endif
C
C
CC----------------------------------------------------------------------
CC
CC Initialize chemical variables 
CC
CC----------------------------------------------------------------------
C
C set pre-industrial atmospheric [co2] (ppm) and o2/n2 ratio
C ----------------------------------------------------------
C
      atcco20 = 278.
      atcox = 0.20946
C
C Set half precision constants
C ----------------------------
C
      tenm7 = 10.**(-7.0)
      smicr = 1.E-6
      thousi = 1./1000.
      perc = 0.01
      fourth = 0.25
      third = 1./3.
      half = 0.5
      one = 1.
      two = 2.
      ten = 10.
C
C Set lower/upper limits for temperature and salinity
C ---------------------------------------------------
C
      tfree = -1.9
      tboil = 30.
      sweet = 20.
      brine = 40.
C
      salchl = 1./1.80655
      temzer = 273.16
      calcon = 1.03E-2
C
C Set coefficients for apparent solubility equilibrium
C   of calcite (Ingle, 1800, eq. 6)
C ----------------------------------------------------
C
      akcc1 = -34.452
      akcc2 = -39.866
      akcc3 = 110.21
      akcc4 = -7.5752E-6
C
      arafra = 0.
      calfra = 1.-arafra
      sucall = arafra+calfra
      aracal = arafra*1.45+calfra
C
C Set coefficients for seawater pressure correction
C -------------------------------------------------
C
      devk1  = 24.2
      devk2  = 16.4
      devkb  = 27.5
      devk1t = 0.085
      devk2t = 0.04
      devkbt = 0.095
C
      devkst = 0.23
      devks  = 32.8*arafra+35.4*calfra
C
C Set universal gas constants
C ---------------------------
C
      rgas = 83.143
      oxyco = 1./22.4144
C
C Set boron constants
C -------------------
C
      bor1 = 0.00023
      bor2 = 1./10.82
C
C Set volumetric solubility constants for co2 in ml/l (Weiss, 1974)
C -----------------------------------------------------------------
C
      c00 = -58.0931
      c01 = 90.5069
      c02 = 22.2940
      c03 = 0.027766
      c04 = -0.025888
      c05 = 0.0050578
C
C Set coeff. for 1. dissoc. of carbonic acid (Edmond and Gieskes, 1970)
C ---------------------------------------------------------------------
C
      c10 = -2307.1266
      c11 = 2.83655
      c12 = -1.5529413
      c13 = -4.0484
      c14 = -0.20760841
      c15 = 0.08468345
      c16 = -0.00654208
      c17 = -0.001005
C
C Set coeff. for 2. dissoc. of carbonic acid (Edmond and Gieskes, 1970)
C ---------------------------------------------------------------------
C
      c20 = -3351.6106
      c21 = -9.226508
      c22 = -0.2005743
      c23 = -23.9722
      c24 = -0.106901773
      c25 = 0.1130822
      c26 = -0.00846934
      c27 = -0.001005
C
C Set coeff. for 1. dissoc. of boric acid (Edmond and Gieskes, 1970)
C ------------------------------------------------------------------
C
      cb0  = -8966.90
      cb1  = -2890.53
      cb2  = -77.942
      cb3  = 1.728
      cb4  = -0.0996
      cb5  = 148.0248
      cb6  = 137.1942
      cb7  = 1.62142
      cb8  = -24.4344
      cb9  = -25.085
      cb10 = -0.2474
      cb11 = 0.053105
C
C Set coeff. for dissoc. of water (Dickson and Riley, 1979, 
C   eq. 7, coefficient cw2 corrected from 0.9415 to 0.09415 
C   after pers. commun. to B. Bacastow, 1988)
C ---------------------------------------------------------
C
      cw0 = -13847.26
      cw1 = 148.9652
      cw2 = -23.6521
      cw3 = 118.67
      cw4 = -5.977
      cw5 = 1.0495
      cw6 = -0.01615
C
C Set volumetric solubility constants for o2 in ml/l (Weiss, 1970)
C ----------------------------------------------------------------
C
      ox0 = -58.3877
      ox1 = 85.8079
      ox2 = 23.8439
      ox3 = -0.034892
      ox4 = 0.015568
      ox5 = -0.0019387
C      
#if defined key_off_degrad
C
C Read volume for degraded regions (DEGINIT)
C ------------------------------------------
C
#    if defined key_vpp
      CALL READ3S(902,facvol,jpi,jpj,jpk)
#    else
      READ (902) facvol
#    endif
#endif
C
C    CALCUL DU MASK DE COTE
C
        cmask=0.
        do ji=2,jpi-1
          do jj=2,jpj-1
            if (tmask(ji,jj,1).eq.1) then
             ztest=tmask(ji+1,jj,1)*tmask(ji-1,jj,1)*tmask(ji,jj+1,1)
     .             *tmask(ji,jj-1,1)
             if (ztest.eq.0) cmask(ji,jj)=1.
             endif
          end do
        end do

        cmask(1,:)=cmask(jpi-1,:)
        cmask(jpi,:)=cmask(2,:)
C
C     CALCUL DE LA SURFACE COTIERE
C
         do ji=2,jpi-1
          do jj=2,jpj-1
          areacot=areacot+e1t(ji,jj)*e2t(ji,jj)*cmask(ji,jj)
          end do
         end do

C
C
      CALL h3cche
      WRITE(numout,*) ' Initialisation of HAMOCC3 done'
